<?php

namespace Drupal\tarte_au_citron\Form;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\tarte_au_citron\LibraryJsDiscoverInterface;
use Drupal\tarte_au_citron\ServicesManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements an abstract form.
 */
abstract class AbstractForm extends ConfigFormBase {

  /**
   * The tarte_au_citron cache bin.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The service manager.
   *
   * @var \Drupal\tarte_au_citron\ServicesManagerInterface
   */
  protected $servicesManager;

  /**
   * The library js discover service.
   *
   * @var \Drupal\tarte_au_citron\ServicesManagerInterface
   */
  protected $libraryJsDiscover;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    ModuleHandlerInterface $module_handler,
    ServicesManagerInterface $servicesManager,
    LibraryJsDiscoverInterface $libraryJsDiscover,
    CacheBackendInterface $cache,
  ) {
    parent::__construct($config_factory);
    $this->moduleHandler = $module_handler;
    $this->servicesManager = $servicesManager;
    $this->libraryJsDiscover = $libraryJsDiscover;
    $this->cache = $cache;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('module_handler'),
      $container->get('tarte_au_citron.services_manager'),
      $container->get('tarte_au_citron.library_js_discover'),
      $container->get('tarte_au_citron.services_js'),
    );
  }

}
